@@ -152,14 +152,16 @@ uint64_t dr_ste_get_icm_addr(struct dr_ste *ste)
 {
 	uint32_t index = ste - ste->htbl->ste_arr;
 
-	return ste->htbl->chunk->icm_addr + DR_STE_SIZE * index;
+	return dr_icm_pool_get_chunk_icm_addr(ste->htbl->chunk) +
+	       DR_STE_SIZE * index;
 }
 
 uint64_t dr_ste_get_mr_addr(struct dr_ste *ste)
 {
 	uint32_t index = ste - ste->htbl->ste_arr;
 
-	return ste->htbl->chunk->mr_addr + DR_STE_SIZE * index;
+	return dr_icm_pool_get_chunk_mr_addr(ste->htbl->chunk) +
+	       DR_STE_SIZE * index;
 }
 
 struct list_head *dr_ste_get_miss_list(struct dr_ste *ste)
@@ -195,7 +197,7 @@ static void dr_ste_always_hit_htbl(struct dr_ste_ctx *ste_ctx,
 	ste_ctx->set_ctrl_always_hit_htbl(hw_ste,
 					  next_htbl->byte_mask,
 					  next_htbl->lu_type,
-					  chunk->icm_addr,
+					  dr_icm_pool_get_chunk_icm_addr(chunk),
 					  chunk->num_of_entries,
 					  gvmi);
 
@@ -245,8 +247,15 @@ dr_ste_remove_head_ste(struct dr_ste_ctx *ste_ctx,
 	uint8_t formated_ste[DR_STE_SIZE] = {};
 	struct dr_htbl_connect_info info;
 
+	stats_tbl->ctrl.num_of_valid_entries--;
+
+	/* Hash table will be deleted, no need to update STE */
+	if (atomic_load(&ste->htbl->refcount) == 1)
+		return;
+
 	info.type = CONNECT_MISS;
-	info.miss_icm_addr = nic_matcher->e_anchor->chunk->icm_addr;
+	info.miss_icm_addr =
+		dr_icm_pool_get_chunk_icm_addr(nic_matcher->e_anchor->chunk);
 	dr_ste_set_formated_ste(ste_ctx,
 				dmn->info.caps.gvmi,
 				nic_dmn->type,
@@ -263,8 +272,6 @@ dr_ste_remove_head_ste(struct dr_ste_ctx *ste_ctx,
 					      ste_info_head,
 					      send_ste_list,
 					      true /* Copy data */);
-
-	stats_tbl->ctrl.num_of_valid_entries--;
 }
 
 /*
@@ -420,7 +427,9 @@ void dr_ste_set_hit_addr_by_next_htbl(struct dr_ste_ctx *ste_ctx,
 {
 	struct dr_icm_chunk *chunk = next_htbl->chunk;
 
-	ste_ctx->set_hit_addr(hw_ste, chunk->icm_addr, chunk->num_of_entries);
+	ste_ctx->set_hit_addr(hw_ste,
+			      dr_icm_pool_get_chunk_icm_addr(chunk),
+			      chunk->num_of_entries);
 }
 
 void dr_ste_prepare_for_postsend(struct dr_ste_ctx *ste_ctx,
@@ -502,7 +511,8 @@ int dr_ste_create_next_htbl(struct mlx5dv_dr_matcher *matcher,
 
 		/* Write new table to HW */
 		info.type = CONNECT_MISS;
-		info.miss_icm_addr = nic_matcher->e_anchor->chunk->icm_addr;
+		info.miss_icm_addr =
+			dr_icm_pool_get_chunk_icm_addr(nic_matcher->e_anchor->chunk);
 		if (dr_ste_htbl_init_and_postsend(dmn, nic_dmn, next_htbl,
 						  &info, false, send_ring_idx)) {
 			dr_dbg(dmn, "Failed writing table to HW\n");
@@ -677,8 +687,9 @@ static int dr_ste_alloc_modify_hdr_chunk(struct mlx5dv_dr_action *action,
 	if (!action->rewrite.param.chunk)
 		return ENOMEM;
 
-	action->rewrite.param.index = (action->rewrite.param.chunk->icm_addr -
-		action->rewrite.dmn->info.caps.hdr_modify_icm_addr) /
+	action->rewrite.param.index =
+		(dr_icm_pool_get_chunk_icm_addr(action->rewrite.param.chunk) -
+		 action->rewrite.dmn->info.caps.hdr_modify_icm_addr) /
 		ACTION_CACHE_LINE_SIZE;
 
 	ret = dr_send_postsend_action(action->rewrite.dmn, action);
@@ -822,11 +833,14 @@ int dr_ste_build_ste_arr(struct mlx5dv_dr_matcher *matcher,
 static void dr_ste_copy_mask_misc(char *mask, struct dr_match_misc *spec, bool clear)
 {
 	spec->gre_c_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_c_present, clear);
+	spec->bth_a = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, bth_a, clear);
 	spec->gre_k_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_k_present, clear);
 	spec->gre_s_present = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_s_present, clear);
 	spec->source_vhca_port = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_vhca_port, clear);
 	spec->source_sqn = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_sqn, clear);
 
+	spec->source_eswitch_owner_vhca_id =
+		DEVX_GET(dr_match_set_misc, mask, source_eswitch_owner_vhca_id);
 	spec->source_port = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, source_port, clear);
 
 	spec->outer_second_prio = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_prio, clear);
@@ -844,30 +858,44 @@ static void dr_ste_copy_mask_misc(char *mask, struct dr_match_misc *spec, bool c
 		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_second_svlan_tag, clear);
 	spec->inner_second_svlan_tag =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_second_svlan_tag, clear);
-
+	spec->outer_emd_tag = DEVX_GET(dr_match_set_misc, mask, outer_emd_tag);
+	spec->reserved_at_65 = DEVX_GET(dr_match_set_misc, mask, reserved_at_65);
 	spec->gre_protocol = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_protocol, clear);
 
 	spec->gre_key_h = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_key_h, clear);
 	spec->gre_key_l = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, gre_key_l, clear);
 
 	spec->vxlan_vni = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, vxlan_vni, clear);
+	spec->bth_opcode = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, bth_opcode, clear);
 
 	spec->geneve_vni = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_vni, clear);
+	spec->reserved_at_e4 = DEVX_GET(dr_match_set_misc, mask, reserved_at_e4);
 	spec->geneve_oam = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_oam, clear);
+	spec->reserved_at_ec = DEVX_GET(dr_match_set_misc, mask, reserved_at_ec);
 	spec->geneve_tlv_option_0_exist =
 		DEVX_GET(dr_match_set_misc, mask, geneve_tlv_option_0_exist);
 
 	spec->outer_ipv6_flow_label =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, outer_ipv6_flow_label, clear);
 
+	spec->reserved_at_100 = DEVX_GET(dr_match_set_misc, mask, reserved_at_100);
 	spec->inner_ipv6_flow_label =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, inner_ipv6_flow_label, clear);
 
+	spec->reserved_at_120 = DEVX_GET(dr_match_set_misc, mask, reserved_at_120);
 	spec->geneve_opt_len = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_opt_len, clear);
 	spec->geneve_protocol_type =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, geneve_protocol_type, clear);
 
+	spec->reserved_at_140 = DEVX_GET(dr_match_set_misc, mask, reserved_at_140);
 	spec->bth_dst_qp = DR_DEVX_GET_CLEAR(dr_match_set_misc, mask, bth_dst_qp, clear);
+
+	spec->inner_esp_spi = DEVX_GET(dr_match_set_misc, mask, inner_esp_spi);
+	spec->outer_esp_spi = DEVX_GET(dr_match_set_misc, mask, outer_esp_spi);
+
+	spec->reserved_at_1a0 = DEVX_GET(dr_match_set_misc, mask, reserved_at_1a0);
+	spec->reserved_at_1c0 = DEVX_GET(dr_match_set_misc, mask, reserved_at_1c0);
+	spec->reserved_at_1e0 = DEVX_GET(dr_match_set_misc, mask, reserved_at_1e0);
 }
 
 static void dr_ste_copy_mask_spec(char *mask, struct dr_match_spec *spec, bool clear)
@@ -895,6 +923,7 @@ static void dr_ste_copy_mask_spec(char *mask, struct dr_match_spec *spec, bool c
 	spec->tcp_sport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, tcp_sport, clear);
 	spec->tcp_dport = DR_DEVX_GET_CLEAR(dr_match_spec, mask, tcp_dport, clear);
 
+	spec->reserved_at_c0 = DEVX_GET(dr_match_spec, mask, reserved_at_c0);
 	spec->ipv4_ihl = DR_DEVX_GET_CLEAR(dr_match_spec, mask, ipv4_ihl, clear);
 	spec->l3_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, l3_ok, clear);
 	spec->l4_ok = DR_DEVX_GET_CLEAR(dr_match_spec, mask, l4_ok, clear);
@@ -965,6 +994,9 @@ static void dr_ste_copy_mask_misc2(char *mask, struct dr_match_misc2 *spec, bool
 	spec->metadata_reg_c_1 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_1, clear);
 	spec->metadata_reg_c_0 = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_c_0, clear);
 	spec->metadata_reg_a = DR_DEVX_GET_CLEAR(dr_match_set_misc2, mask, metadata_reg_a, clear);
+	spec->reserved_at_1a0 = DEVX_GET(dr_match_set_misc2, mask, reserved_at_1a0);
+	spec->reserved_at_1c0 = DEVX_GET(dr_match_set_misc2, mask, reserved_at_1c0);
+	spec->reserved_at_1e0 = DEVX_GET(dr_match_set_misc2, mask, reserved_at_1e0);
 }
 
 static void dr_ste_copy_mask_misc3(char *mask, struct dr_match_misc3 *spec, bool clear)
@@ -973,12 +1005,16 @@ static void dr_ste_copy_mask_misc3(char *mask, struct dr_match_misc3 *spec, bool
 	spec->outer_tcp_seq_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_tcp_seq_num, clear);
 	spec->inner_tcp_ack_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, inner_tcp_ack_num, clear);
 	spec->outer_tcp_ack_num = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_tcp_ack_num, clear);
+
+	spec->reserved_at_80 = DEVX_GET(dr_match_set_misc3, mask, reserved_at_80);
 	spec->outer_vxlan_gpe_vni =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_vni, clear);
 	spec->outer_vxlan_gpe_next_protocol =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_next_protocol, clear);
 	spec->outer_vxlan_gpe_flags =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, outer_vxlan_gpe_flags, clear);
+	spec->reserved_at_b0 = DEVX_GET(dr_match_set_misc3, mask, reserved_at_b0);
+
 	spec->icmpv4_header_data = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmp_header_data, clear);
 	spec->icmpv6_header_data =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmpv6_header_data, clear);
@@ -988,13 +1024,15 @@ static void dr_ste_copy_mask_misc3(char *mask, struct dr_match_misc3 *spec, bool
 	spec->icmpv6_code = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, icmpv6_code, clear);
 	spec->geneve_tlv_option_0_data =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, geneve_tlv_option_0_data, clear);
-	spec->gtpu_msg_flags = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_flags, clear);
-	spec->gtpu_msg_type = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_type, clear);
 	spec->gtpu_teid = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_teid, clear);
-	spec->gtpu_dw_0 = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_dw_0, clear);
+	spec->gtpu_msg_type = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_type, clear);
+	spec->gtpu_msg_flags = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_msg_flags, clear);
+	spec->reserved_at_170 = DEVX_GET(dr_match_set_misc3, mask, reserved_at_170);
 	spec->gtpu_dw_2 = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_dw_2, clear);
 	spec->gtpu_first_ext_dw_0 =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_first_ext_dw_0, clear);
+	spec->gtpu_dw_0 = DR_DEVX_GET_CLEAR(dr_match_set_misc3, mask, gtpu_dw_0, clear);
+	spec->reserved_at_1e0 = DEVX_GET(dr_match_set_misc3, mask, reserved_at_1e0);
 }
 
 static void dr_ste_copy_mask_misc4(char *mask, struct dr_match_misc4 *spec, bool clear)
@@ -1051,6 +1089,22 @@ static void dr_ste_copy_mask_misc5(char *mask, struct dr_match_misc5 *spec, bool
 		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_2, clear);
 	spec->tunnel_header_3 =
 		DR_DEVX_GET_CLEAR(dr_match_set_misc5, mask, tunnel_header_3, clear);
+	spec->reserved_at_100 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_100);
+	spec->reserved_at_120 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_120);
+	spec->reserved_at_140 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_140);
+	spec->reserved_at_160 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_160);
+	spec->reserved_at_180 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_180);
+	spec->reserved_at_1a0 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_1a0);
+	spec->reserved_at_1c0 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_1c0);
+	spec->reserved_at_1e0 =
+		DEVX_GET(dr_match_set_misc5, mask, reserved_at_1e0);
 }
 
 void dr_ste_copy_param(uint8_t match_criteria,
@@ -1478,6 +1532,19 @@ void dr_ste_build_tunnel_header(struct dr_ste_ctx *ste_ctx,
 	ste_ctx->build_tunnel_header_init(sb, mask);
 }
 
+void dr_ste_build_ib_l4(struct dr_ste_ctx *ste_ctx,
+			struct dr_ste_build *sb,
+			struct dr_match_param *mask,
+			bool inner, bool rx)
+{
+	if (!ste_ctx->build_ib_l4_init)
+		return;
+
+	sb->rx = rx;
+	sb->inner = inner;
+	ste_ctx->build_ib_l4_init(sb, mask);
+}
+
 int dr_ste_build_def0(struct dr_ste_ctx *ste_ctx,
 		      struct dr_ste_build *sb,
 		      struct dr_match_param *mask,
